severity=5
#SEVERITY NAME   ...is equivalent to...   SEVERITY NUMBER
#--------------------------------------------------------
#-severity => 'gentle'                     -severity => 5
#-severity => 'stern'                      -severity => 4
#-severity => 'harsh'                      -severity => 3
#-severity => 'cruel'                      -severity => 2
#-severity => 'brutal'                     -severity => 1

# we use Fatal qw(open close)
#[-InputOutput::RequireCheckedOpen]
#[-InputOutput::RequireCheckedClose]

[Documentation::RequirePodSections]
source   = module_starter_pbp_0_0_3
language = en_US

[Miscellanea::RequireRcsKeywords]
keywords = Date Author Revision
