# -*- perl -*-

# /t/500_full_swarm/501_1-to-9.t - Miniature swarm of 1 seed and 9 new peers
# $Id: 503_1-to-9.t 10 2008-04-05 22:06:00Z sanko@cpan.org $

system qq[$^X ./t/data/etc/http_miniswarm.pl 1 9];
