# -*- perl -*-

# /t/500_full_swarm/501_5-to-5.t - Miniature swarm of 5 seeds and 5 new peers
# $Id: 506_5-to-5.t 10 2008-04-05 22:06:00Z sanko@cpan.org $

system qq[$^X ./t/data/etc/http_miniswarm.pl 5 5];
